;******************************************************************************
;
;       minirgb 16F84
;       Program = RGB LED Controller with menu
;       Revision Date: 13.04.07
;       Version 1.6
;       
;
;******************************************************************************
;******************************************************************************
;******************************************************************************
;=========================
;=       INFO            =
;=========================


;******************************************************************************
;
;       Quarz 20 MHz
;
;******************************************************************************

;******************************************************************************
;******************************************************************************
;=========================
;=   PORT CONNECTIONS    =
;=========================


;******************************************************************************
;       
;       PORTA0 = R
;       PORTA1 = G
;       PORTA2 = B
;       PORTA3 = Anode of 7seg
;       PORTA4 = buttons
;
;******************************************************************************

;******************************************************************************
;       
;       PORTB0 = 7seg a / button l
;       PORTB1 = 7seg b / button m 
;       PORTB2 = 7seg c / button r
;       PORTB3 = 7seg dp
;       PORTB4 = 7seg d
;       PORTB5 = 7seg e
;       PORTB6 = 7seg g
;       PORTB7 = 7seg f
;
;******************************************************************************

;******************************************************************************
;******************************************************************************

;=========================
;=        DEFINES        =
;=========================

;******************************************************************************
;F=Flag
;B=Bitvar for bit defines
;S=static
;T=temporary
;LA=low active       
;P=pointer var
;ADR=adressdefine
;BUT=buttondefine
;F_TL1_=temporary bit in stack level 1 function (no use in L2 function call)
;C = const
;ASM = assembler define
;******************************************************************************

;********************************************************** HARDWARE WRAP LAYER
;******************************************************************************

;************************************************************* HARTDWARE STRUCT
;******************************************************************************
;
;                                    VCC
;  minirgb circuit             ____   +             RGB LED
;                             |    |  |
;    +-+------o----------o----|7805|--o          |       |       |
;    A A      | +        |    |____|  |       ||-+    ||-+    ||-+
; ~ -+ |     ### 1000uF ---     |    --- .----||<-  .-||<-  .-||<-
; ~ -(-+     ---        ---     |    --- |    ||-+  | ||-+  | ||-+
;    A A      |     100nF|      |100nF|  |       |  |    |  |    | BS170
;    +-+------o----------o------o-----o--)-------o--)----o--)----' n-ch
;                                        |          |       |
; VCC       .----------------------------'          |       |        VCC
;  +--.   .-)---------------------------------------)-------)----.    +
;  |  |   | | .---------.                           |       |    |    |
; .-..-.  | | |         | PIC16F84A 20M             |       |    |    |
; |2||1|  | '-o PA2 RA1 o---------------------------'       |    |    |
;K|2||K|  |   |         |                                   |    |  |-+
; '-''-'  '---o PA3 RA0 o-----------------------------------'    '-<|
;  |  |       |         |                                           |-+
;  o--)-------o PA4  Q2 o-------o-------||---- GND                    |
;  |  |       |         | 20Mhz '-|H|-. 22n |                   BS250 |
;  |  '-------o R    Q1 o-------------o-||--'   HDSP F 101      p-ch  |
;  |          |         |                     .---------------.       |
;  |   .------o GND VCC o----|VCC             o +   -----   A o----.  |
;  |   |      |         |                     |    |     |    |    |  |
;  |   | .----o RB0 RB7 o---------------------o F  |     |  B o---.| .-.
;  |   | |    |         |                     |    |     |    |   || |3|
;  |   | |.---o RB1 RB6 o---------------------o G   -----   C o--.|| |3|
;  |   | ||   |         |                     |    |     |    |  ||| |0|
;  |   | ||.--o RB2 RB5 o---------------------o E  |     | DP o-.||| '-'
;  |   | |||  |         |                     |    |     |    | ||||  |
;  |   | |||.-o RB3 RB4 o---------------------o D   ----- o + o-))))--'
;  |   | |||| |         |                     |               | ||||
;  |   | |||| '---------'                     '---------------' ||||
;  |   | |||'---------------------------------------------------'|||
;  |   | ||'-------------------------------------o---------------'||
;  |   | |'-------------------------o------------)----------------'|
;  |   | '-------------o------------)------------)-----------------'
;  |   |               |            |            |
;  |  ===              |            |            |
;  |  GND              -            -            -
;  |                   ^            ^            ^ 1N4148
;  |                   |            |            |
;  |                   |            |            |
;  |                 | o          | o          | o
;  |         Down  |=|>   Menu  |=|>     Up  |=|>
;  |                 | |          | o          | o
;  |                   |            |            |
;  '-------------------o------------o------------'
;
;******************************************************************************
;******************************************************* END OF HARDWARE STRUCT

;********** Port A Defines
;*************************

#Define         PWMCH1          PORTA,0         ;PWM Ch1
#Define         PWMCH2          PORTA,1         ;PWM Ch2
#Define         PWMCH3          PORTA,2         ;PWM Ch3
#Define         ANODE           PORTA,3         ;Common anode of display (fet)
#Define         HWBUTREV        PORTA,4         ;button rev of the 3 buttons 
                                                ;(if change - pin is input!)


;********** Port B Defines
;*************************

#Define         SEGDAT          PORTB           ;
#Define         SEGDATDIR       TRISB           ;

#Define         HWBUTL          PORTB,0         ;button left connected to:
#Define         HWBUTM          PORTB,1         ;button middle connected to:
#Define         HWBUTR          PORTB,2         ;button right connected to:

#Define         HWSEGA          PORTB,0         ;segment a connected to:
#Define         HWSEGB          PORTB,1         ;segment b connected to:
#Define         HWSEGC          PORTB,2         ;segment c connected to:
#Define         HWSEGDP         PORTB,3         ;segment dp connected to:
#Define         HWSEGD          PORTB,4         ;segment d connected to:
#Define         HWSEGE          PORTB,5         ;segment e connected to:
#Define         HWSEGG          PORTB,6         ;segment f connected to:
#Define         HWSEGF          PORTB,7         ;segment g connected to:

;************* PWM defines
;*************************

#Define         ASM_LOW         bcf             ;
#Define         ASM_HIGH        bsf             ;

#Define         ASM_ON          ASM_HIGH        ;on->high - n-channel (for p-channel change here)
#Define         ASM_OFF         ASM_LOW         ;off->low - n-channel (for p-channel change here)

;******************************************************************************
;****************************************************** END HARDWARE WRAP LAYER

;** Hardware control Flags
;*************************

#Define         F_1HZTRIG       HWCTRL,0        ;1hz trigger from int
#Define         HWBIT1          HWCTRL,1        ;nu
#Define         F_HWDISPDP      HWCTRL,2        ;dot point on/off
#Define         F_HWDISPMAIN    HWCTRL,3        ;Display on/off (not used->off=h'20' ascii space)
#Define         F_PWMHNDL       HWCTRL,4        ;handle flag between pwm routines
#Define         F_PL            HWCTRL,5        ;prell help bits left
#Define         F_PM            HWCTRL,6        ;middle
#Define         F_PR            HWCTRL,7        ;right

;** Software control Flags
;*************************

#Define         MNU0            SWCTRL,0        ;
#Define         MNU1            SWCTRL,1        ;for menu dep setting changes and show param
#Define         BUTTONL         SWCTRL,2        ;virtual button left
#Define         BUTTONM         SWCTRL,3        ;virtual button middle
#Define         BUTTONR         SWCTRL,4        ;virtual button right
#Define         F_SAVE          SWCTRL,5        ;L=one time save param in 1hz grid
#Define         F_ON            SWCTRL,6        ;L=one time display on in 1hz grid
#Define         F_SHOW          SWCTRL,7        ;L=one time show menu in 1hz grid

;****** Additional defines
;*************************

;** 7 seg decoder Flags
;*************************

#Define         DECSEGA         VAL,6           ;
#Define         DECSEGB         VAL,7           ;
#Define         DECSEGC         VAL,1           ;
#Define         DECSEGD         VAL,2           ;
#Define         DECSEGE         VAL,3           ;
#Define         DECSEGF         VAL,5           ;
#Define         DECSEGG         VAL,4           ;
#Define         DECSEGDP        VAL,0           ;

;********************************7seg decoder**********************************
;
;       Bit0 dp/nc
;       Bit1 c
;       Bit2 d
;       Bit3 e
;       
;       Bit4 g
;       Bit5 f
;       Bit6 a
;       Bit7 b
;       
;
;****************************************************************************** 


;************ DSIPLAY Defs
;*************************


;******************************************************************************
;******************************************************************************
;******************************************************************************
;=========================
;=        MACROS         =
;=========================


;******************************************************************************

;************** Bankselect
;*************************

BANK0_          macro
                bcf             STATUS,5
                bcf             STATUS,6
                endm

BANK1_          macro
                bsf             STATUS,5
                bcf             STATUS,6
                endm

;*************** Interrupt
;*************************

PUSH_           macro
                movwf           ISR_W
                swapf           STATUS,w
                movwf           ISR_S
                endm

POP_            macro
                swapf           ISR_S,w
                movwf           STATUS
                swapf           ISR_W,f
                swapf           ISR_W,w
                endm

;******************************************************************************
;******************************************************************************
;******************************************************************************

;=========================
;=        CODE           =
;=========================



;*************************************************************** Include  Files 
;******************************************************************************

  LIST    P=PIC16F84
#INCLUDE "P16F84.INC"
;#define __16F84


;******************************************************************** Variables 
;******************************************************************************

;******************* const
;*************************

C_ISRTIME       equ     h'05'   ; 5* 1/5 sec (20mhz) = 1sec        
C_PRELLT        equ     h'F9'   ;prellzeit, fr taster
C_SAVET         equ     h'3c'   ;sicherungzeit (in s) bis speicherung in ee passiert(muss grer als menzeigezeit sein)
C_ONT           equ     h'3c'   ;time display (in s) is on(muss grer als menzeigezeit sein)
C_SHOWT         equ     h'01'   ;Time to show menu var

;****** Standard Variables
;*************************

HWCTRL          equ     h'0c'   ;
SWCTRL          equ     h'0d'   ;help bits
ISR_W           equ     h'0e'   ;
ISR_S           equ     h'0f'   ;isr bits

;*********** LCD Variables
;*************************


;********** WAIT Variables
;*************************

TIME1           equ     h'10'   ;
TIME2           equ     h'11'   ;
TIME3           equ     h'12'   ;
TIME4           equ     h'13'   ;counter in main


;********* Other Variables
;*************************

ISR_TMR         equ     h'14'   ;prescaler in interrupt
ISR_TMR1        equ     h'15'   ;
VAL             equ     h'16'   ;
DEC7S           equ     h'17'   ;

RISE            equ     h'18'
UP              equ     h'19'   ;
FALL            equ     h'1a'   ;
DOWN            equ     h'1b'   ;
SPEED           equ     h'1c'   ;
SHIFT           equ     h'1d'
PROG            equ     h'1e'

MENU            equ     h'1f'   ;


VARIABLE        LPWM=h'20'      ;

PWM1ADR         equ     LPWM    ;indirect adressed, dont change drection of this block
PWM1CNT         equ     LPWM    ;
LPWM++                          ;
PWM1CTRL        equ     LPWM    ;
LPWM++                          ;
PWM1            equ     LPWM    ;
LPWM++                          ;
PWM1HOLD        equ     LPWM    ;
LPWM++                          ;

PWM2ADR         equ     LPWM    ;indirect adressed, dont change drection of this block
PWM2CNT         equ     LPWM    ;
LPWM++                          ;
PWM2CTRL        equ     LPWM    ;
LPWM++                          ;
PWM2            equ     LPWM    ;
LPWM++                          ;
PWM2HOLD        equ     LPWM    ;
LPWM++                          ;

PWM3ADR         equ     LPWM    ;indirect adressed, dont change drection of this block
PWM3CNT         equ     LPWM    ;
LPWM++                          ;
PWM3CTRL        equ     LPWM    ;
LPWM++                          ;
PWM3            equ     LPWM    ;
LPWM++                          ;
PWM3HOLD        equ     LPWM    ;
LPWM++                          ;


PWMOK           equ     LPWM    ;if there is a rotating bit in pwmcntrl (helpvar for restart)
LPWM++                          ;
PWMNEXT         equ     LPWM    ;save the pwm modul range for SHIFT func to start next bit
LPWM++                          ;
 
VARIABLE        LUSE=LPWM       ; 

HWDISPADR       equ     LUSE    ;
HWDISP          equ     LUSE    ;
LUSE++
HWMUX           equ     LUSE    ;
LUSE++

ADREE           equ     LUSE    ;
LUSE++
DATAEE          equ     LUSE    ;
LUSE++

SAVE            equ     LUSE    ;
LUSE++
ON              equ     LUSE    ;
LUSE++
SHOW            equ     LUSE    ;
LUSE++

PL              equ     LUSE    ;
LUSE++
PR              equ     LUSE    ;
LUSE++
PM              equ     LUSE    ;
LUSE++

;************************************************************** SOFTWARE STRUCT
;******************************************************************************
;.----------------------------------------------------------------------.
;|                                                                      |
;|           MAIN : _START     (software  structure of minirgb)         |
;|                                                                      |
;'-oo-------o----------o----------o----------o---------------------o--o-'
;  ||       |          |          |          |                     |  |
;  ||       |          |          |          |                     |  |
;  ||.------o---. .----o----. .---o---. .----o----.                |  |
;  |||_HWMUX    | |_TIMINGS | |_PRELL | |_PWMCTRL |                |  |
;  |||          | |         | |       | |         |                |  |
;  |||          | |         | |       | |         |                |  |
;  |||         3| |        3| |      3| |        3|                |  |
;  ||'------o---' '-o-o-o---' '---o---' '-o-o-o-o-'                |  |
;  ||       |       | | |         |       | | | |                  |  |
;  ||       |       | | |         |       | | | |                  |  |
;  ||       |       | | |         |       | | | |                  |  |
;  ||       |       | | |         |       | | | |                  |  |
;  ||       |       | | |         |       | | | |                  |  |
;  ||       |       | | |         |       | | | |                  |  |
;  ||       |       | | | .-------)-------' | | |                  |  |
;  ||       |       | | | |       |  -------' | |                  |  |
;  ||       |       | | | |       | |         | |                  |  |
;  ||       |       | | '-)-------)-)---------)-)----------.       |  |
;  ||       |       | |   |       | |         | |          |       |  |
;  ||       |       | '---)-----. | | .-------)-)----------|-------'  |
;  ||       |       |     |     | | | |       | |          |          |
;  ||       |       |   .-)-----)-' | |       | |          |          |
;  ||       |       |   | |     |   | |       | |          |          |
;  ||.------o---. .-o---o-o-. .-o---o-o-.     | |  .-------o-. .------o-.
;  |||_DECODE   | |_MENUSEL | |_CNTRL   |     | |  |_WRITE   | |_READ   |
;  |||          | |         | |SETTINGS |     | |  |SETTINGS | |SETTINGS|
;  |||          | |         | |         |     | |  |         | |        |
;  |||         2| |        2| |        2|     | |  |        2| |       2|
;  ||'------o---' '-o-------' '-o-o-----'     | |  '-------o-' '------o-'
;  ||       |       |           | |           | |          |          |
;  ||       |       |           | |           | |          |          |
;  ||       |       |           | |           | |          |          |
;  ||       |       |           | |           | |          |          |
;  ||       |       |     .-----' |           | |          |          |
;  ||       |       |     |       |           | |          |          |
;  ||       |       |     |       |           | |          |          |
;  ||       |       |     |       |   .-------' |          |          |
;  ||       |    0-3|    0|       |   |         |          |          |
;  ||       |       |     |       |   |         |          |          |
;.-oo--. .--o---. .-o-----o-. .---o---o-. .-----o-----. .--o----. .---o-.
;|_PWM | |_7SEG | |_LOAD-   | |_CLR_PWM | |_PWMMODUL- | |_WRITE | |_READ|
;|     | |      | |DEFAULTS | |         | |CTRL       | |EE     | |EE   |
;|    1| |     1| |0-3     1| |        1| |          1| |      1| |    1|
;'-----' '------' '---------' '---------' '-----------' '-------' '-----'
;******************************************************************************
;******************************************************* END OF SOFTWARE STRUCT

;****************************************************** Restet/Interrupt Vektor
;******************************************************************************
org             .0
                goto            _INIT           ;Reset Vektor
org             .4
                goto            _ISR            ;Interrupt Service Routine

;******************************************************************* INIT START
;******************************************************************************
_INIT           
                BANK1_                          ;change to Bank 1

;Port A
 
                movlw           b'00010000'     ;0->Output
                movwf           TRISA           ;
 
;Port B
 
                movlw           b'00000000'     ;0->Output
                movwf           TRISB           ;
 
;Option Reg 

                movlw           b'00000011'     ;Prescaler on(Bit3) 0 bis 2 Prescaler(auf 2^4(teilung 16)
                movwf           OPTION_REG      ; 

;Intcon Reg
     
                movlw           b'10100000'     ;Interruptkontrolle bit5 tmr0 int on/off, bit4 rb0 int on/off
                movwf           INTCON          ;

                BANK0_                          ;change to Bank 0

;************************************************************** Initialisierung
;******************************************************************************

                clrf            HWCTRL          ;control bits =0
                clrf            SWCTRL          ;

;                clrf            RISE            ;enthlt anstiegszeit
;                clrf            UP              ;enthlt haltezeit ein
;                clrf            FALL            ;enthlt abfallzeit
;                clrf            DOWN            ;enthlt haltezeit aus
;                clrf            SPEED           ;enthlt geschwindigkeit
;clear in readee clrf            SHIFT           ;enthlt Versatz zwischen kanlen
         
                clrf            PWM1
                clrf            PWM2
                clrf            PWM3

                clrf            MENU            ;start by showing menu 0
                clrf            PROG            ;enthlt aktuelles programm
                
                clrf            SHOW            ;time to show menu =0 in first on


;***************************************** ONE TIMES CALLED FUNCTIONS AND INITS
;******************************************************************************

                movlw           C_ISRTIME
                movwf           ISR_TMR1

                call            _CLR_PWM

                bsf             F_HWDISPMAIN    ;switch on display
       
                bsf             F_SAVE          ;gewrleistet keine speicherung beim einschalten
                bsf             F_ON            ;gewrleistet kein off bis ON abgelaufen ist
                bcf             F_SHOW          ;gewrleistet das die anzeige aktiv ist beim ersten einschalten
                                                ;bewirkt im aufruf von timings, in abhngigkeit des 1 hz triggers, die 
                                                ;initialisierung des displays, speed wert anzeigen
                bsf             F_1HZTRIG

                movlw           C_ONT
                movwf           ON
                movlw           C_SAVET         ;savetime / starts saving count down
                movwf           SAVE            ;bewirkt bei leerem ee prom, ersten einschalten, 
                                                ;das speichern der default werte als startsetting
            
                bcf             BUTTONR
                bcf             BUTTONM
                bcf             BUTTONL


                call            _READSETTINGS   ;read settings from last use
                call            _CNTRLSETTINGS  ;control settings (if there is a wron reat value)

                movlw           h'30'
                movwf           HWDISP

 
;**************************************************************** START OF MAIN
;******************************************************************************

_START
                
                movlw           .10
                movwf           TIME4


_LOOP4          movlw           .10
                movwf           TIME3


_LOOP3          incf            SPEED,w
                movwf           TIME2

                call            _PWMCTRL    ;PWM Control Unit

_LOOP2          movlw           .99
                movwf           TIME1
 
                call            PRELL_      ;Tastenentprellung

                call            TIMINGS_    ;generate timings ->poll:base 1hz

               ;call            HWWRAP_     ;possible but not imp.

                call            HWMUX_                               
		
_LOOP1          goto            _PWM        ;
_BPWM           decfsz          TIME1,F	
                goto            _LOOP1
                decfsz          TIME2,F
                goto            _LOOP2
                decfsz          TIME3,F
                goto            _LOOP3
                decfsz          TIME4,F
                goto            _LOOP4

_END            goto            _START

;******************************************************************************
;****************************************************************** END OF MAIN

;************************************************************************ INITS
;******************************************************************************




;******************************************************************************
;***************************************************************** END OF INITS

;*************************************************************** MAIN FUNCTIONS
;******************************************************************************

                ;**************************************
                ;*                                    *
                ;*           *
                ;*                                    *
                ;************************************** 

                ;***************
                ;*             *
                ;*    *
                ;*             *
                ;***************

;------------------------------------------------------------------------------
;Cat:         SW Func / _PWMCTRL
;------------------------------------------------------------------------------
;Level:       x
;------------------------------------------------------------------------------
;Task:        
;------------------------------------------------------------------------------
;Functions:   
;Based on:    
;------------------------------------------------------------------------------
;Vars:        
;Internal:    
;------------------------------------------------------------------------------
;Bits:        
;Internal:    
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------


;******************************************************************************
;*********************************************************** END MAIN FUNCTIONS
;******************************************************************************
;*************************************************************** CLASS xxx



;------------------------------------------------------------------------------
;Cat:         SW Func / class xx / func xxx
;------------------------------------------------------------------------------
;Level:       x
;------------------------------------------------------------------------------
;Task:        
;------------------------------------------------------------------------------
;Functions:
;  USE:
;    Public:      no
;                 
;                 
;                 
;    Private:     no
;  BASE:
;    Public:      no
;                 
;                 
;                 
;                 
;                 
;    Private:     no
;------------------------------------------------------------------------------
;Vars:            no
;Internal:        no    
;------------------------------------------------------------------------------
;Bits:
;  USE:
;    Public:      no                
;    Private:     no           
;  BASE:          
;    Public:      no
;    Private:     no
;                 no
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------


;*********************************************************** END CLASS xxx
;******************************************************************************

;******************************************************************** FUNCTIONS
;******************************************************************************

;------------------------------------------------------------------------------
;Cat:         HW Func / PRELL
;------------------------------------------------------------------------------
;Level:       3
;------------------------------------------------------------------------------
;Task:        do action if button is pushed
;------------------------------------------------------------------------------
;Functions:   PRELL_
;called Func: _MENUSEL  2
;------------------------------------------------------------------------------
;Vars:        HWDISP
;             PL
;             PR
;             PM
;------------------------------------------------------------------------------
;Bits:        BUTTONL
;             BUTTONM
;             BUTTONR           ;virtual button of mux routines
;internal     F_PL
;             F_PM
;             F_PR              ;help bits
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------


PRELL_
                btfsc           BUTTONL
                goto            PRELL_L_        ;Prell left

                btfsc           BUTTONM
                goto            PRELL_M_        ;Prell middle

                btfsc           BUTTONR
                goto            PRELL_R_        ;Prell right

                goto            PRELL_NO_       ;no prell

;***********************************************Taste Mitte**********

PRELL_M_                                        ;PRELL mitte
                clrf            PL
                clrf            PR

                bcf             F_PR            ;right
                bcf             F_PL            ;left

                bcf             STATUS,0
                movf            PM,w
                addlw           C_PRELLT        ;time
                btfsc           STATUS,0
                goto            PM_Y
                incf            PM,f
                goto            PM_N

PM_Y            bcf             STATUS,0
                btfsc           F_PM
                goto            PM_N
                bsf             F_PM

;*******************************Aktion Button middle ****************

                bsf             F_HWDISPDP      ;menu has been selected, dot point on
                
                incf            MENU,f          ;makes menu navigation possible
        
                bcf             STATUS,0
                movf            MENU,w
                addlw           h'F8'           ;menpunkte von 0 bis 7
                btfss           STATUS,0
                goto            NO_M_
                movlw           h'00'
                movwf           MENU
NO_M_           bcf             STATUS,0

                movf            MENU,w
                addlw           h'30'
                movwf           HWDISP          ;show menu val
                ;***********************

                movlw           C_SHOWT         ;showtime / starts showing menu var countdown
                movwf           SHOW

                movlw           C_ONT           ;time display is on
                movwf           ON

                ;bei menpunkt 6 muss zustzlich noch eine speicherung erfolgen

                bcf             STATUS,2
                movf            MENU,w
                addlw           h'FA'           ;abfrage auf Menu ==6(PROG)
                btfss           STATUS,2
                goto            MENU_MDL_PASS_
                bcf             STATUS,2

                movlw           C_SAVET         ;savetime / starts saving count down
                movwf           SAVE


MENU_MDL_PASS_
;*******************************End Button middle ********************
PM_N
                goto            PRELL_E_        ;PRELL EXIT

;***********************************************Taste Rechts**********

PRELL_R_                                        ;PRELL rechts

                clrf            PL
                clrf            PM

                bcf             F_PL            ;left
                bcf             F_PM            ;middle

                bcf             STATUS,0
                movf            PR,w
                addlw           C_PRELLT
                btfsc           STATUS,0
                goto            PR_Y
                incf            PR,f
                goto            PR_N

PR_Y            bcf             STATUS,0
                btfsc           F_PR
                goto            PR_N
                bsf             F_PR

;*******************************Aktion Button right  ******************

                bsf             MNU0
                bcf             MNU1            ;count up settings

                call            _MENUSEL
                
;*******************************End Button right ********************
PR_N
                goto            PRELL_LR_                 ;PRELL EXIT

;***********************************************Taste Links**********

PRELL_L_                                        ;PRELL links
                clrf            PR
                clrf            PM

                bcf             F_PR            ;right
                bcf             F_PM            ;middle

                
                bcf             STATUS,0
                movf            PL,w
                addlw           C_PRELLT
                btfsc           STATUS,0
                goto            PL_Y
                incf            PL,f
                goto            PL_N

PL_Y            bcf             STATUS,0
                btfsc           F_PL
                goto            PL_N
                bsf             F_PL

;*******************************Aktion Button left *******************
        
                bcf             MNU0
                bcf             MNU1            ;count down settings

                call            _MENUSEL
        
;*******************************End Button left **********************
PL_N
                goto            PRELL_LR_       ;PRELL EXIT

;*******************************used from Prell left a. right********
PRELL_LR_
                clrf            SHOW            ;disables menu show, if the button is earlier pushed as menu has been showed
                bcf             F_SHOW          ;just one show of settings
                bcf             F_HWDISPDP      ;dot point off

                movlw           C_SAVET         ;savetime / starts saving count down
                movwf           SAVE

                movlw           C_ONT           ;time display is on
                movwf           ON

                goto            PRELL_E_        ;PRELL left right EXIT
;*********************************************************************

PRELL_NO_
                clrf            PR
                clrf            PL
                clrf            PM

                bcf             F_PR            ;right
                bcf             F_PL            ;left
                bcf             F_PM            ;middle


;*****************************************END OF PRELLS****************
PRELL_E_        
                return

;------------------------------------------------------------------------------
;Cat:         SW Func / TIMINGS
;------------------------------------------------------------------------------
;Level:       3
;------------------------------------------------------------------------------
;Task:        control timings for Save/Display Off/show parameter (settings)
;------------------------------------------------------------------------------
;Functions:   _TIMINGS
;called Func: _CNTRLSETTINGS
;             _WRITESETTINGS
;             _MENUSEL             
;------------------------------------------------------------------------------
;Vars:        HWDISP
;Int:         SAVE
;             ON
;             SHOW
;------------------------------------------------------------------------------
;Bits:        F_1HZTRIG       ;1Hz Trigger from Int
;             F_HWDISPDP      ;dot point in the display
;             F_SHOW          ;SW SHOW BIT
;Int:         
;             F_SAVE          ;SW SAVE BIT
;             F_ON            ;SW ON BIT
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------


TIMINGS_
                btfss           F_1HZTRIG       ;1hz trigger from interrupt
                goto            TIMINGEND_
                bcf             F_1HZTRIG       ;else goto end...
                                                ;if one second has been gone:

                ;**************************************
                ;*                                    *
                ;*              Save Timing           *
                ;*                                    *
                ;**************************************
               

                bcf             STATUS,2
                movf            SAVE,w
                addlw           h'00'
                btfss           STATUS,2        ;is save time ==0
                goto            NOSAVE_         ;no
                                                ;yes
                btfsc           F_SAVE
                goto            ENDSAVE_

                ;*******Aktion save timing******
                
                ;call            _CNTRLSETTINGS  ;control settings before 
                                                ;saving...
                                                ;if switch off happens in 
                                                ;this moment...
                
                call            _WRITESETTINGS  ;saving call

                ;*******************************
                
                bsf             F_SAVE          ;close one save
                goto            ENDSAVE_

NOSAVE_         decf            SAVE,f          ;saving time goes down
                bcf             F_SAVE          ;open one save after time SAVET is down
ENDSAVE_        bcf             STATUS,2

               

                ;**************************************
                ;*                                    *
                ;*         display off timing         *
                ;*                                    *
                ;**************************************

                bcf             STATUS,2
                movf            ON,w
                addlw           h'00'
                btfss           STATUS,2
                goto            NOOFF_          ;wenn on != 00
                                                ;wenn on == 00

                btfsc           F_ON  
                goto            ENDOFF_         ;wenn gesperrt
                                                ;wenn nicht gesperrt

                ;**Aktion display off***********

                movlw           h'20'           ;display off
                movwf           HWDISP

                ;*******************************
                
                bsf             F_ON            ;sperren (freigabe erfolgt bei zuweisung on=wert)
                goto            ENDOFF_

NOOFF_          decf            ON,f            ;saving time goes down
                bcf             F_ON            ;freigabe fr einmal off
ENDOFF_         bcf             STATUS,2



                ;**************************************
                ;*                                    *
                ;*           Show menu timing         *
                ;*                                    *
                ;**************************************

                bcf             STATUS,2
                movf            SHOW,w
                addlw           h'00'
                btfss           STATUS,2
                goto            SHOWMENU_
                                                ;show one time a setting var/depends on ctrl 3 /if set no refresh
                btfsc           F_SHOW
                goto            SHOWEND_

                bsf             F_SHOW          ;sperrt einmaliges zeigen der settings

                ;***Aktion menudep.*************

                bcf             MNU0
                bsf             MNU1            ;diplay settings

                call            _MENUSEL

                bcf             F_HWDISPDP      ;schaltet den men-punkt (dp in 7 seg) aus
                goto            SHOWEND_

                ;*******************************

SHOWMENU_       decf            SHOW,f
                bcf             F_SHOW          ;gibt einmaliges anzeigen der settings frei
                movf            MENU,w
                addlw           h'30'
                movwf           HWDISP
SHOWEND_

                ;End...

TIMINGEND_
                return
;------------------------------------------------------------------------------
;Cat:         SW Func / _PWMCTRL
;------------------------------------------------------------------------------
;Level:       3
;------------------------------------------------------------------------------
;Task:        generate the dimm flow between the channels
;------------------------------------------------------------------------------
;Functions:   _PWMCTRL
;called func:
;             PWMMODULCTRL_
;             _CLR_PWM
;             _CNTRLSETTINGS        
;             _MENUSEL     
;------------------------------------------------------------------------------
;Vars:        
;Internal:    
;------------------------------------------------------------------------------
;Bits:        
;Internal:    
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------
_PWMCTRL
                bsf             PWMOK,0
                

                
                movlw           PWM1ADR
                call            PWMMODULCTRL_

                movlw           PWM2ADR
                call            PWMMODULCTRL_

                movlw           PWM3ADR
                call            PWMMODULCTRL_

       

                btfsc           PWMOK,0         ;berprfung und reaktion, geht bit verlohren 
                bsf             F_PWMHNDL       ;bewirkt neubeginn, rounding bit wird gesetzt

                bcf             STATUS,0        ;prfen ob zu viele bits routieren
                movf            PWMOK,w
                addlw           h'F0'           ;if there more than 3 bits rotating -> reastart pwm an change shift val
                btfss           STATUS,0
                goto            PWMCTRLEND_

                call            _CLR_PWM

                incf            SHIFT,f         ;funktion zum zhlen in Down von 0-7

                call            _CNTRLSETTINGS        
        
                bcf             MNU0
                bsf             MNU1            ;diplay settings

                call            _MENUSEL        ;kann nur bei nderungen an den settings 1-4 aufgerufen werden
PWMCTRLEND_     clrf            PWMOK
                return


;------------------------------------------------------------------------------
;Cat:         SW Func / _MENUSEL
;------------------------------------------------------------------------------
;Level:       2 because of default loadings
;------------------------------------------------------------------------------
;Task:        do up and down counts of the pwm parameter, and show these,
;             depending on the two bits BIT 2,1 and var:MENU
;------------------------------------------------------------------------------
;Functions:   _MENUSEL
;called Func: _LOADDEFAULTS0
;             _LOADDEFAULTS1
;             _LOADDEFAULTS2
;             _LOADDEFAULTS3
;             _CLR_PWM
;------------------------------------------------------------------------------
;Vars:        MENU                              var, global swicht
;             SPEED             0 -> SPEED
;             RISE              1 -> RISE
;             UP                2 _> UP
;             FALL              3 -> FALL
;             DOWN              4 -> DOWN
;             SHIFT             5 -> SHIFT
;             PROG              6 -> 
;                               7 -> RESET PWM ( call _CLR_PWM)
;             
;             HWDISP (contains value of show parameter) (is used in HW Layer as
;                                                     display var)
;Internal:    no
;------------------------------------------------------------------------------
;Bits:        MNU1  MNU0
;             0     0     count down setting (selected in menu)
;             0     1     count up   setting (selected in menu)
;             1     0     move selected setting to HWDISP
;Internal:    no
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------


_MENUSEL

;****************************SPEED*********************************************

                bcf             STATUS,2
                movf            MENU,w
                addlw           h'00'           ;abfrage auf Menu ==0(SPEED)
                btfss           STATUS,2
                goto            SPEED_NO


                btfsc           MNU1 
                goto            MENU_DISPSPEED_

                btfsc           MNU0
                goto            MENU_UPSPEED_
                goto            MENU_DWNSPEED_



SPEED_UD_
MENU_DISPSPEED_ movf            SPEED,w         ;Display refresh
                addlw           h'30'           ;select ascii numbers
                movwf           HWDISP
                goto            MENU_PASS_

MENU_UPSPEED_   incf            SPEED,f         ;funktion zum zhlen in speed von 0-9
                movf            SPEED,w
                bcf             STATUS,0
                addlw           h'F6'
                btfss           STATUS,0
                goto            SPEED_UD_
                movlw           h'00'
                movwf           SPEED
                goto            SPEED_UD_
                
MENU_DWNSPEED_  decf            SPEED,f         ;funktion zum zhlen in Speed von 9-0
                movf            SPEED,w
                bcf             STATUS,0
                addlw           h'01'
                btfss           STATUS,0
                goto            SPEED_UD_
                movlw           h'09'
                movwf           SPEED
                goto            SPEED_UD_

;****************************RISE*********************************************                
SPEED_NO
                bcf             STATUS,2
                movf            MENU,w
                addlw           h'FF'           ;abfrage auf Menu ==1(RISE)
                btfss           STATUS,2
                goto            RISE_NO



                btfsc           MNU1 
                goto            MENU_DISPRISE_

                btfsc           MNU0
                goto            MENU_UPRISE_
                goto            MENU_DWNRISE_


RISE_UD_                
MENU_DISPRISE_  movf            RISE,w
                addlw           h'30'           ;select ascii numbers
                movwf           HWDISP
                goto            MENU_PASS_

MENU_UPRISE_    incf            RISE,f          ;funktion zum zhlen in Rise von 0-9
                movf            RISE,w
                bcf             STATUS,0
                addlw           h'F6'
                btfss           STATUS,0
                goto            RISE_UD_
                movlw           h'00'
                movwf           RISE
                goto            RISE_UD_
                
MENU_DWNRISE_   decf            RISE,f          ;funktion zum zhlen in Rise von 9-0
                movf            RISE,w
                bcf             STATUS,0
                addlw           h'01'
                btfss           STATUS,0
                goto            RISE_UD_
                movlw           h'09'
                movwf           RISE
                goto            RISE_UD_

;****************************UP***********************************************        
RISE_NO
                bcf             STATUS,2
                movf            MENU,w
                addlw           h'FE'           ;abfrage auf Menu ==2(UP)
                btfss           STATUS,2
                goto            UP_NO
 
                btfsc           MNU1 
                goto            MENU_DISPUP_

                btfsc           MNU0
                goto            MENU_UPUP_
                goto            MENU_DWNUP_

UP_UD_                
MENU_DISPUP_    movf            UP,w
                addlw           h'30'           ;select ascii numbers
                movwf           HWDISP
                goto            MENU_PASS_

MENU_UPUP_      incf            UP,f            ;funktion zum zhlen in Up von 0-9
                movf            UP,w
                bcf             STATUS,0
                addlw           h'F6'
                btfss           STATUS,0
                goto            UP_UD_
                movlw           h'00'
                movwf           UP
                goto            UP_UD_
                
MENU_DWNUP_     decf            UP,f            ;funktion zum zhlen in Up von 9-0
                movf            UP,w
                bcf             STATUS,0
                addlw           h'01'
                btfss           STATUS,0
                goto            UP_UD_
                movlw           h'09'
                movwf           UP
                goto            UP_UD_

;****************************FALL*********************************************        
UP_NO
                bcf             STATUS,2
                movf            MENU,w
                addlw           h'FD'           ;abfrage auf Menu ==3(FALL)
                btfss           STATUS,2
                goto            FALL_NO


                btfsc           MNU1
                goto            MENU_DISPFALL_

                btfsc           MNU0
                goto            MENU_UPFALL_
                goto            MENU_DWNFALL_

FALL_UD_                
MENU_DISPFALL_  movf            FALL,w
                addlw           h'30'           ;select ascii numbers
                movwf           HWDISP
                goto            MENU_PASS_

MENU_UPFALL_    incf            FALL,f          ;funktion zum zhlen in Fall von 0-9
                movf            FALL,w
                bcf             STATUS,0
                addlw           h'F6'
                btfss           STATUS,0
                goto            FALL_UD_
                movlw           h'00'
                movwf           FALL
                goto            FALL_UD_
                
MENU_DWNFALL_   decf            FALL,f          ;funktion zum zhlen in Fall von 9-0
                movf            FALL,w
                bcf             STATUS,0
                addlw           h'01'
                btfss           STATUS,0
                goto            FALL_UD_
                movlw           h'09'
                movwf           FALL
                goto            FALL_UD_

;****************************DOWN*********************************************        
FALL_NO
                bcf             STATUS,2
                movf            MENU,w
                addlw           h'FC'           ;abfrage auf Menu ==4(DOWN)
                btfss           STATUS,2
                goto            DOWN_NO

                btfsc           MNU1
                goto            MENU_DISPDOWN_

                btfsc           MNU0
                goto            MENU_UPDOWN_
                goto            MENU_DWNDOWN_

DOWN_UD_
MENU_DISPDOWN_  movf            DOWN,w
                addlw           h'30'           ;select ascii numbers
                movwf           HWDISP
                goto            MENU_PASS_

MENU_UPDOWN_    incf            DOWN,f          ;funktion zum zhlen in Down von 0-9
                movf            DOWN,w
                bcf             STATUS,0
                addlw           h'F6'
                btfss           STATUS,0
                goto            DOWN_UD_
                movlw           h'00'
                movwf           DOWN
                goto            DOWN_UD_
                
MENU_DWNDOWN_   decf            DOWN,f          ;funktion zum zhlen in Down von 9-0
                movf            DOWN,w
                bcf             STATUS,0
                addlw           h'01'
                btfss           STATUS,0
                goto            DOWN_UD_
                movlw           h'09'
                movwf           DOWN
                goto            DOWN_UD_

;****************************SHIFT********************************************                
DOWN_NO
                bcf             STATUS,2
                movf            MENU,w
                addlw           h'FB'           ;abfrage auf Menu ==5(SHIFT)
                btfss           STATUS,2
                goto            SHIFT_NO

                btfsc           MNU1
                goto            MENU_DISPSFT_

                btfsc           MNU0
                goto            MENU_UPSFT_
                goto            MENU_DWNSFT_


SHIFT_UD_
MENU_DISPSFT_   movf            SHIFT,w
                addlw           h'30'           ;select ascii numbers
                movwf           HWDISP
                goto            MENU_PASS_

MENU_UPSFT_     incf            SHIFT,f         ;funktion zum zhlen in Down von 0-7
                movf            SHIFT,w
                bcf             STATUS,0
                addlw           h'F8'
                btfss           STATUS,0
                goto            SHIFT_UD_
                movlw           h'00'
                movwf           SHIFT
                goto            SHIFT_UD_
                
MENU_DWNSFT_    decf            SHIFT,f         ;funktion zum zhlen in Down von 7-0
                movf            SHIFT,w
                bcf             STATUS,0
                addlw           h'01'
                btfss           STATUS,0
                goto            SHIFT_UD_
                movlw           h'07'
                movwf           SHIFT
                goto            SHIFT_UD_

;****************************PROG*********************************************                
SHIFT_NO
                bcf             STATUS,2
                movf            MENU,w
                addlw           h'FA'           ;abfrage auf Menu ==6(PROG)
                btfss           STATUS,2
                goto            PROG_NO_

                btfsc           MNU1
                goto            MENU_DISPPROG_

                btfsc           MNU0
                goto            MENU_UPPROG_
                goto            MENU_DWNPROG_


MENU_DISPPROG_
                movf            PROG,w
                addlw           h'30'           ;select ascii numbers
                movwf           HWDISP
                goto            MENU_PASS_


MENU_UPPROG_    incf            PROG,f          ;funktion zum zhlen in Down von 0-3
                movf            PROG,w
                bcf             STATUS,0
                addlw           h'FC'
                btfss           STATUS,0
                goto            PROG_UD_
                movlw           h'00'
                movwf           PROG
                goto            PROG_UD_

                
MENU_DWNPROG_   decf            PROG,f          ;funktion zum zhlen in Down von 3-0
                movf            PROG,w
                bcf             STATUS,0
                addlw           h'01'
                btfss           STATUS,0
                goto            PROG_UD_
                movlw           h'03'
                movwf           PROG
                goto            PROG_UD_
PROG_UD_
                call            _CLR_PWM
                btfss           PROG,1
                goto            PROG_LDF01_
                goto            PROG_LDF23_

PROG_LDF01_     btfss           PROG,0
                goto            PROG_LDF0_
                call            _LOADDEFAULTS1
                goto            MENU_DISPPROG_
PROG_LDF0_      call            _LOADDEFAULTS0
                goto            MENU_DISPPROG_


PROG_LDF23_     btfss           PROG,0
                goto            PROG_LDF2_
                call            _LOADDEFAULTS3
                goto            MENU_DISPPROG_
PROG_LDF2_      call            _LOADDEFAULTS2
                goto            MENU_DISPPROG_

;****************************CLR*********************************************                
PROG_NO_
                bcf             STATUS,2
                movf            MENU,w
                addlw           h'F9'           ;abfrage auf Menu ==7(CLR)
                btfss           STATUS,2
                goto            MENU_PASS_


                call            _CLR_PWM
                goto            MENU_PASS_


;*****************************END MENU****************************************        
MENU_PASS_
                return

;------------------------------------------------------------------------------
;Cat:         SW Func / LOADDEFAULTS
;------------------------------------------------------------------------------
;Level:       1
;------------------------------------------------------------------------------
;Task:        set the parameter vars to predefined values
;------------------------------------------------------------------------------
;Functions:   _LOADDEFAULTS0 -
;             _LOADDEFAULTS3
;called Func: no
;------------------------------------------------------------------------------
;Vars:        SPEED
;             RISE
;             UP
;             FALL
;             DOWN
;             SHIFT
;------------------------------------------------------------------------------
;Bits:        no
;             
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------


_LOADDEFAULTS0
                movlw           h'04'           ;menu 0
                movwf           SPEED
        
                movlw           h'09'           ;menu 1
                movwf           RISE
        
                movlw           h'09'           ;menu 2
                movwf           UP

                movlw           h'09'           ;menu 3
                movwf           FALL

                movlw           h'00'           ;menu 4
                movwf           DOWN

                movlw           h'02'           ;menu 5    (0-7) 0=rise halbe (mindest abstand von lampe zu lampe) 7=down (max abstand)
                movwf           SHIFT

                
                return

_LOADDEFAULTS1

                movlw           h'03'           ;menu 0
                movwf           SPEED
        
                movlw           h'09'           ;menu 1
                movwf           RISE
        
                movlw           h'05'           ;menu 2
                movwf           UP

                movlw           h'09'           ;menu 3
                movwf           FALL

                movlw           h'05'           ;menu 4
                movwf           DOWN

                movlw           h'01'           ;menu 5    (0-7) 0=rise halbe (mindest abstand von lampe zu lampe) 7=down (max abstand)
                movwf           SHIFT
 
                return


_LOADDEFAULTS2
                movlw           h'05'           ;menu 0
                movwf           SPEED
        
                movlw           h'09'           ;menu 1
                movwf           RISE
        
                movlw           h'03'           ;menu 2
                movwf           UP

                movlw           h'03'           ;menu 3
                movwf           FALL

                movlw           h'09'           ;menu 4
                movwf           DOWN

                movlw           h'01'           ;menu 5    (0-7) 0=rise halbe (mindest abstand von lampe zu lampe) 7=down (max abstand)
                movwf           SHIFT

        
                return


_LOADDEFAULTS3
                movlw           h'01'           ;menu 0
                movwf           SPEED
        
                movlw           h'09'           ;menu 1
                movwf           RISE
        
                movlw           h'09'           ;menu 2
                movwf           UP

                movlw           h'09'           ;menu 3
                movwf           FALL

                movlw           h'00'           ;menu 4
                movwf           DOWN

                movlw           h'05'           ;menu 5    (0-7) 0=rise halbe (mindest abstand von lampe zu lampe) 7=down (max abstand)
                movwf           SHIFT

        
                return

;------------------------------------------------------------------------------
;Cat:         SW Func / CLR_PWM
;------------------------------------------------------------------------------
;Level:       1
;------------------------------------------------------------------------------
;Task:        clear the hole pwm settings
;------------------------------------------------------------------------------
;Functions:   no
;called Func: no
;------------------------------------------------------------------------------
;Vars:        ->
;------------------------------------------------------------------------------
;Bits:        no
;             no
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------


_CLR_PWM
                clrf            PWM1            ;
                clrf            PWM2            ;
                clrf            PWM3            ;
 
                clrf            PWM1CNT         ;enthalten die jeweiligen zeiten eines abschnittes (RISE,UP,FALL,DOWN) einer PWM (1-6)
                clrf            PWM2CNT         ;
                clrf            PWM3CNT         ;
          
                clrf            PWM1CTRL        ;enthlt ablausteuerungsbits fr den flankenverlauf des dimmens
                clrf            PWM1HOLD        ;speichert den wert den der ausgang GANZ aus, bzw an ist und gehalten wird

                clrf            PWM2CTRL        ;
                clrf            PWM2HOLD        ;

                clrf            PWM3CTRL        ;
                clrf            PWM3HOLD        ;

                clrf            PWMNEXT         ;bit array, anthlt momentanen pwm zustand, (rise halbe,rise,up halbe,up,...) 
                                                ;wird mit shift in pwm mod verglichen

                clrf            PWMOK           ;controls the current of pwm

                return                


;------------------------------------------------------------------------------
;Cat:         SW Func / PWMMODULCTRL_
;------------------------------------------------------------------------------
;Level:       1
;------------------------------------------------------------------------------
;Task:        generate the rise/hold/fall/down info in var: PWM (indirect func)
;------------------------------------------------------------------------------
;Functions:   PWMMODULCTRL_ 
;Based on:    no
;------------------------------------------------------------------------------
;Vars:        pointer to struct pwm has to be in w at call
;             PWMNEXT           ;help for SHIFT func
;             PWMOK             ;to indicate the activity of a phase
;             PWM1       (ind.) ;ch1 pwm (value handl betw. HWPWM & control)
;             PWM2       (ind.) ;ch2 pwm (value handl betw. HWPWM & control)
;             PWM3       (ind.) ;ch3 pwm (value handl betw. HWPWM & control)
;Internal:    PWMCNT1-3  (ind.) ;scale vars for streching the phases
;             PWMCTRL1-3 (ind.) ;flow control of func
;             PWMHOLD1-3 (ind.) ;used for the phases UP/DOWN (no PWM used)
;------------------------------------------------------------------------------
;Bits:        F_PWMHNDL         ;activate and initialize a channel
;Internal:    
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------
PWMMODULCTRL_ 
                movwf           FSR
                incf            FSR,f          
      
                btfsc           F_PWMHNDL
                bsf             INDF,7

RISEINIT_       btfss           INDF,7
                goto            UPINIT_

                bcf             F_PWMHNDL

                bcf             INDF,7
                bsf             INDF,3
                decf            FSR,f
                movf            RISE,w
                movwf           INDF
                incf            INDF,f
                incf            FSR,f            
                                      
                
UPINIT_         btfss           INDF,6
                goto            FALLINIT_

                bcf             INDF,6
                bsf             INDF,2
                decf            FSR,f
                movf            UP,w
                movwf           INDF
                incf            INDF,f
                incf            FSR,f

FALLINIT_       btfss           INDF,5
                goto            DOWNINIT_

                bcf             INDF,5
                bsf             INDF,1
                decf            FSR,f
                movf            FALL,w
                movwf           INDF
                incf            INDF,f
                incf            FSR,f

DOWNINIT_       btfss           INDF,4
                goto            RISE_

                bcf             INDF,4
                bsf             INDF,0
                decf            FSR,f
                movf            DOWN,w
                movwf           INDF
                incf            INDF,f
                incf            FSR,f





RISE_           btfss           INDF,3
                goto            UP_

                decf            FSR,f           ;point to PWMCNT

                decfsz          INDF,f
                goto            PWMMODULENDCNT_ ;nchste pwm prfen

                incf            FSR,f           ;point to PWMCTRL
 
                bsf             INDF,7          ;bewirkt neuladen von PWMCNT im nchsten durchgang

                incf            FSR,f           ;point to PWM

                incf            INDF,f          ;eigendlicher dimm vorgang

                ;bcf             STATUS,2        ;test ob dimm ist halb max
                movf            INDF,w
                addlw           h'CE'
                btfss           STATUS,2
                goto            GOON_RISE_      ;SHIFT (rise halbe)
                movlw           h'01'
                movwf           PWMNEXT                        
                goto            PWMMODULENDPWM_
GOON_RISE_                                      ;point to PWM

                ;bcf             STATUS,C        ;test ob dimm ist max
                movf            INDF,w
                addlw           h'9D'
                btfss           STATUS,C
                goto            PWMMODULENDPWM_ ;(noch keine volle Helligkeit)nchste pwm prfen
                                                ;(volle Helligkeit erreicht) bergang in nchste phase
                decf            FSR,f

                bcf             INDF,7          ;letztes neuladen rckganig machen

                bcf             INDF,3          ;Rise abgeschlossen

                bsf             INDF,6          ;einleitung der nchsten Phase

                movlw           h'02'
                movwf           PWMNEXT         ;SHIFT (rise)

                goto            PWMMODULEND_    ;nchste pwm prfen



        

UP_             btfss           INDF,2
                goto            FALL_

                decf            FSR,f           ;point to PWMCNT

                decfsz          INDF,f
                goto            PWMMODULENDCNT_ ;nchste pwm prfen

                incf            FSR,f           ;point to PWMCTRL

                bsf             INDF,6          ;bewirkt neuladen von PWMCNT im nchsten durchgang

                incf            FSR,f           ;point to PWM
                incf            FSR,f           ;point to PWMHOLD                 

                incf            INDF,f          ;eigendlicher halte vorgang

                ;bcf             STATUS,2        ;test ob dimm ist halb max
                movf            INDF,w
                addlw           h'CE'
                btfss           STATUS,2
                goto            GOON_UP_        ;SHIFT (up halbe)
                movlw           h'03'
                movwf           PWMNEXT                        
                goto            PWMMODULENDHLD_
GOON_UP_

                ;bcf             STATUS,C        ;test ob hold ist max
                movf            INDF,w
                addlw           h'9D'
                btfss           STATUS,C
                goto            PWMMODULENDHLD_ ;(noch keine 10 stufen gehalten)nchste pwm prfen
                                                ;(volle 10 stufen gehalten) bergang in nchste phase
                clrf            INDF            ;haltevariable lschen

                decf            FSR,f           ;point to PWM
                decf            FSR,f           ;point to PWMCTRL
                
                bcf             INDF,6          ;letztes neuladen rckganig machen

                bcf             INDF,2          ;Up abgeschlossen

                bsf             INDF,5          ;einleitung der nchsten Phase

                movlw           h'04'
                movwf           PWMNEXT         ;SHIFT (up)


                goto            PWMMODULEND_    ;nchste pwm prfen





FALL_           btfss           INDF,1
                goto            DOWN_

                decf            FSR,f           ;point to PWMCNT

                decfsz          INDF,f
                goto            PWMMODULENDCNT_ ;nchste pwm prfen

                incf            FSR,f           ;point to PWMCTRL

                bsf             INDF,5          ;bewirkt neuladen von PWMCNT im nchsten durchgang

                incf            FSR,f           ;point to PWM

                decf            INDF,f          ;eigendlicher dimm vorgang

                ;bcf             STATUS,2        ;test ob dimm ist halb max
                movf            INDF,w
                addlw           h'CE'
                btfss           STATUS,2
                goto            GOON_FALL_      ;SHIFT (fall halbe)
                movlw           h'05'
                movwf           PWMNEXT                        
                goto            PWMMODULENDPWM_
GOON_FALL_

                ;bcf             STATUS,Z        ;test ob dimm ist min
                movf            INDF,w
                addlw           h'00'
                btfss           STATUS,Z
                goto            PWMMODULENDPWM_ ;(noch keine volle Dunkelheit)nchste pwm prfen
                                                ;(volle Dunkelheit erreicht) bergang in nchste phase

                decf            FSR,f           ;point to PWMCTRL               

                bcf             INDF,5          ;letztes neuladen rckganig machen

                bcf             INDF,1          ;Fall abgeschlossen

                bsf             INDF,4          ;einleitung der nchsten Phase

                movlw           h'06'
                movwf           PWMNEXT         ;SHIFT (fall)


                goto            PWMMODULEND_    ;nchste pwm prfen




        

DOWN_           btfss           INDF,0
                goto            PWMMODULEND_    ;komplette Phase ist nicht aktiv

                decf            FSR,f           ;point to PWMCNT

                decfsz          INDF,f
                goto            PWMMODULENDCNT_ ;nchste pwm prfen

                incf            FSR,f           ;point to PWMCTRL

                bsf             INDF,4          ;bewirkt neuladen von PWMCNT im nchsten durchgang

                incf            FSR,f           ;
                incf            FSR,f           ;point to PWMHOLD

                incf            INDF,f          ;eigendlicher halte vorgang

                ;bcf             STATUS,2        ;test ob dimm ist halb max
                movf            INDF,w
                addlw           h'CE'
                btfss           STATUS,2
                goto            GOON_DOWN_      ;SHIFT (down halbe)
                movlw           h'07'
                movwf           PWMNEXT                        
                goto            PWMMODULENDHLD_
GOON_DOWN_

                ;bcf             STATUS,C        ;test ob hold ist max
                movf            INDF,w
                addlw           h'9D'
                btfss           STATUS,C
                goto            PWMMODULENDHLD_ ;(noch keine 10 stufen gehalten)nchste pwm prfen
                                                ;(volle 10 stufen gehalten) bergang in nchste phase
                clrf            INDF            ;haltevariable lschen

                decf            FSR,f           ;
                decf            FSR,f           ;point to PWMCTRL
                
                bcf             INDF,4          ;letztes neuladen rckganig machen

                bcf             INDF,0          ;Down abgeschlossen
        
                movlw           h'08'
                movwf           PWMNEXT         ;SHIFT (down)
                goto            PWMMODULEND_




PWMMODULENDHLD_ decf            FSR,f           ;pointer offset corr
PWMMODULENDPWM_ decf            FSR,f           ;pointer offset corr
PWMMODULEND_    decf            FSR,f
PWMMODULENDCNT_ incf            FSR,f
                movf            PWMNEXT,w
                xorlw           h'FF'
                addlw           h'02'
                bcf             STATUS,2
                addwf           SHIFT,w
                btfsc           STATUS,2
                bsf             F_PWMHNDL       ;start next phase
                bcf             STATUS,2
                clrf            PWMNEXT

                                
                bcf             STATUS,Z        ;berprft nur ob der inhalt der ctrl variablen null ist
                movf            INDF,w          ;was wiederum bedeutet das diese phase gerade inaktiv ist...
                addlw           h'00'
                btfss           STATUS,Z
                rlf             PWMOK,f         ;falls in dieser routine eine phase luft, routiert

                return

;------------------------------------------------------------------------------
;Cat:         SW Func / EECONTROL
;------------------------------------------------------------------------------
;Level:       2
;------------------------------------------------------------------------------
;Task:        save the settings in eeprom, an read it back
;------------------------------------------------------------------------------
;Functions:   _READSETTINGS  2
;             _WRITESETTINGS 2
;             _CNTRLSETTINGS 2
;Based on:    _READEE
;             _WRITEEE
;------------------------------------------------------------------------------
;Vars:        ADREE
;             DATAEE
;Internal:    
;------------------------------------------------------------------------------
;Bits:        
;Internal:    
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------


_READSETTINGS
                clrf            ADREE
                call            _READEE
                movf            DATAEE,w
                movwf           SPEED

                incf            ADREE,f
                call            _READEE
                movf            DATAEE,w
                movwf           RISE

                incf            ADREE,f
                call            _READEE
                movf            DATAEE,w
                movwf           UP

                incf            ADREE,f
                call            _READEE
                movf            DATAEE,w
                movwf           FALL

                incf            ADREE,f
                call            _READEE
                movf            DATAEE,w
                movwf           DOWN

                incf            ADREE,f
                call            _READEE
                movf            DATAEE,w
                movwf           SHIFT

                return

_WRITESETTINGS
                clrf            ADREE
                movf            SPEED,w
                movwf           DATAEE
                call            _WRITEEE

                incf            ADREE,f
                movf            RISE,w
                movwf           DATAEE
                call            _WRITEEE

                incf            ADREE,f
                movf            UP,w
                movwf           DATAEE
                call            _WRITEEE

                incf            ADREE,f
                movf            FALL,w
                movwf           DATAEE
                call            _WRITEEE

                incf            ADREE,f
                movf            DOWN,w
                movwf           DATAEE
                call            _WRITEEE

                incf            ADREE,f
                movf            SHIFT,w
                movwf           DATAEE
                call            _WRITEEE

                return

_CNTRLSETTINGS

                bcf             STATUS,0
                movf            SPEED,w
                addlw           h'F6'           ;between 0-9
                btfsc           STATUS,0
                goto            CNTRL_MISTAKE_


                bcf             STATUS,0
                movf            RISE,w
                addlw           h'F6'           ;between 0-9
                btfsc           STATUS,0
                goto            CNTRL_MISTAKE_


                bcf             STATUS,0
                movf            UP,w
                addlw           h'F6'           ;between 0-9
                btfsc           STATUS,0
                goto            CNTRL_MISTAKE_


                bcf             STATUS,0
                movf            FALL,w
                addlw           h'F6'           ;between 0-9
                btfsc           STATUS,0
                goto            CNTRL_MISTAKE_


                bcf             STATUS,0
                movf            DOWN,w
                addlw           h'F6'           ;between 0-9
                btfsc           STATUS,0
                goto            CNTRL_MISTAKE_


                bcf             STATUS,0
                movf            SHIFT,w
                addlw           h'F8'           ;between 0-7
                btfsc           STATUS,0
                goto            CNTRL_MISTAKE_
                goto            CNTRL_MIS_END_
CNTRL_MISTAKE_
                call            _CLR_PWM
                call            _LOADDEFAULTS0
CNTRL_MIS_END_
                return


;*************************************************************** HARDWARE LAYER
;******************************************************************************

;------------------------------------------------------------------------------
;Cat:         HW Func / Mux Layer
;------------------------------------------------------------------------------
;Level:       3
;------------------------------------------------------------------------------
;Task:        Mux 7 seg display and buttons
;------------------------------------------------------------------------------
;Functions:   
;Based on:    
;------------------------------------------------------------------------------
;Vars:        HWDISP            ;ASCII standard
;             
;Internal:    HWMUX
;------------------------------------------------------------------------------
;Bits:        F_HWDISPDP        ;dot point
;             F_HWDISPMAIN      ;main on/off
;Internal:    
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------

HWMUX_  
                incf            HWMUX,f
                btfsc           HWMUX,2
                clrf            HWMUX

                btfss           HWMUX,1
                goto            _HWMUX01

                btfss           HWMUX,0
                goto            _HWMUX2

                ;***************
                ;*             *
                ;*   MUX 3     *
                ;*             *
                ;***************

                movlw           h'ff'
                movwf           SEGDAT 

                bcf             HWBUTR
                nop
                nop
                btfss           HWBUTREV
                goto            _HWMUX3BUTR
                bcf             BUTTONR
                goto            _BHWMUX3BUTR
_HWMUX3BUTR     bsf             BUTTONR
_BHWMUX3BUTR
                bsf             HWBUTR

                goto            _BHWMUX3
_HWMUX2
                ;***************
                ;*             *
                ;*   MUX 2     *
                ;*             *
                ;***************

                movlw           h'ff'
                movwf           SEGDAT 
                   
                bcf             HWBUTM
                nop
                nop
                btfss           HWBUTREV
                goto            _HWMUX2BUTM
                bcf             BUTTONM
                goto            _BHWMUX2BUTM
_HWMUX2BUTM     bsf             BUTTONM
_BHWMUX2BUTM
                bsf             HWBUTM


_BHWMUX3
                goto            _BHWMUX23
_HWMUX01
                btfss           HWMUX,0
                goto            _HWMUX0

                ;***************
                ;*             *
                ;*   MUX 1     *
                ;*             *
                ;***************

                bsf             ANODE           ;switch off anode off 7seg from last round
                movlw           h'FF'
                movwf           SEGDAT          ;switch off all segments
                   
                bcf             HWBUTL
                nop
                nop
                btfss           HWBUTREV
                goto            _HWMUX1BUTL
                bcf             BUTTONL
                goto            _BHWMUX1BUTL
_HWMUX1BUTL     bsf             BUTTONL
_BHWMUX1BUTL
                bsf             HWBUTL


                goto            _BHWMUX1
_HWMUX0
                ;***************
                ;*             *
                ;*   MUX 0     *
                ;*             *
                ;***************
                
                btfss           F_HWDISPMAIN
                goto            _HWMUXDISPOFF

                movf            HWDISP,w
                movwf           VAL
                call            DECODE_

                btfsc           F_HWDISPDP
                goto            _HWMUXDP
                bsf             HWSEGDP
                goto            _BHWMUXDP
_HWMUXDP        bcf             HWSEGDP
_BHWMUXDP

                bcf             ANODE           ;switch on anode of 7 seg

_HWMUXDISPOFF

_BHWMUX1
_BHWMUX23 
                return


;------------------------------------------------------------------------------
;Cat:         HW code / PWM
;------------------------------------------------------------------------------
;Level:       1
;------------------------------------------------------------------------------
;Task:        generate pwm on pins
;------------------------------------------------------------------------------
;Functions:   no : goto _PWM back _BPWM
;Based on:    no
;------------------------------------------------------------------------------
;Vars:        TIME1
;             PWM1
;             PWM2
;             PWM3
;Internal:    no static info
;------------------------------------------------------------------------------
;Bits:        Pwm outputs:      PWMCH1
;                               PWMCH2
;                               PWMCH3
;Internal:    no static info
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------

_PWM                                            ;Fast Loop PWM
                ;bcf             STATUS,0
                movf            TIME1,w
                addlw           h'9C'
                addwf           PWM1,w
                btfss           STATUS,0
                goto            _PWM1OFF
                ASM_ON          PWMCH1          ;on
                goto            _PWM1ON
_PWM1OFF        ASM_OFF         PWMCH1          ;off
_PWM1ON         
                ;bcf             STATUS,0
                movf            TIME1,w
                addlw           h'9C'
                addwf           PWM2,w
                btfss           STATUS,0
                goto            _PWM2OFF
                ASM_ON          PWMCH2
                goto            _PWM2ON
_PWM2OFF        ASM_OFF         PWMCH2
_PWM2ON
                ;bcf             STATUS,0
                movf            TIME1,w
                addlw           h'9C'
                addwf           PWM3,w
                btfss           STATUS,0
                goto            _PWM3OFF
                ASM_ON          PWMCH3
                goto            _PWM3ON
_PWM3OFF        ASM_OFF         PWMCH3
_PWM3ON
                goto            _BPWM

;------------------------------------------------------------------------------
;Cat:         SW Func
;------------------------------------------------------------------------------
;Level:       2
;------------------------------------------------------------------------------
;Task:        translate data from ascii to 7segm & sort by seg connection
;------------------------------------------------------------------------------
;Functions:   DECODE_ 
;called func: _7SEG                             ;array of 7seg data
;------------------------------------------------------------------------------
;Vars:        VAL                               ;input   ascii
;             ;DEC7S                             ;output  7seg
;             output of information over pin defines
;------------------------------------------------------------------------------
;Bits:        no
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------
   
DECODE_ 
                ;addlw           h'30'           ;sel ascii
		
                ;addlw           h'80'           ;+if called with ascii >7F hex
                ;btfss           STATUS,0
                ;addlw           h'80'           ;-if called with ascii >7F hex
                movwf		VAL
                call            _7SEG            ;get 7 seg value 
                movwf           VAL

                ;movlw           h'FF'
                ;movwf           DEC7S
               
                ;bsf             DEC7S,3          ;switch off dp

                btfsc           DECSEGA
                goto            _SEGAH
                bsf             HWSEGA
                goto            _BSEGAL
_SEGAH          bcf             HWSEGA           ;
_BSEGAL

                btfsc           DECSEGB
                goto            _SEGBH
                bsf             HWSEGB
                goto            _BSEGBL
_SEGBH          bcf             HWSEGB           ;
_BSEGBL

                btfsc           DECSEGC
                goto            _SEGCH
                bsf             HWSEGC
                goto            _BSEGCL
_SEGCH          bcf             HWSEGC           ;
_BSEGCL

                btfsc           DECSEGD
                goto            _SEGDH
                bsf             HWSEGD
                goto            _BSEGDL
_SEGDH          bcf             HWSEGD           ;
_BSEGDL

                btfsc           DECSEGE
                goto            _SEGEH
                bsf             HWSEGE
                goto            _BSEGEL
_SEGEH          bcf             HWSEGE           ;
_BSEGEL

                btfsc           DECSEGF
                goto            _SEGFH
                bsf             HWSEGF
                goto            _BSEGFL
_SEGFH          bcf             HWSEGF           ;
_BSEGFL

                btfsc           DECSEGG
                goto            _SEGGH
                bsf             HWSEGG
                goto            _BSEGGL
_SEGGH          bcf             HWSEGG           ;
_BSEGGL

                return

;------------------------------------------------------------------------------
;Cat:         HW Func / 
;------------------------------------------------------------------------------
;Level:       1
;------------------------------------------------------------------------------
;Task:        
;------------------------------------------------------------------------------
;Functions:   
;Based on:    macros BANK1_/BANK0_
;------------------------------------------------------------------------------
;Vars:        ADREE
;             DATAEE
;Internal:
;header:      EEDATA
;             EECON1
;             EECON2
;             EEADR
;             INTCON   
;------------------------------------------------------------------------------
;Bits:        
;Internal:    
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------

_READEE
                movf            ADREE,w
                movwf           EEADR
                
                BANK1_
                bsf             EECON1,0
                BANK0_

                movf            EEDATA,w
                movwf           DATAEE
                return

_WRITEEE
                movf            ADREE,w
                movwf           EEADR
                movf            DATAEE,w
                movwf           EEDATA
                BANK1_
                bcf             INTCON,7
                bsf             EECON1,2        ;WREN bit
                movlw           h'55'           ;required sequence
                movwf           EECON2
                movlw           h'AA'
                movwf           EECON2
                bsf             EECON1,1        ;WR bit ->start write
WRTEEPOLL_      btfss           EECON1,4        ;EEIF poll if write is complete
                goto            WRTEEPOLL_

                clrf            EECON1          ;clear ee control reg.
                BANK0_
                bsf             INTCON,7        ;GIE enable
                return



;******************************************************************************
;*********************************************************** END HARDWARE LAYER

;******************************************************************************
;************************************************************* END OF FUNCTIONS

;************************************************************ INTERRUPT HANDLER
;******************************************************************************

;********************************************************** INTERRUPT FUNCTIONS
;******************************************************************************


_ISR                                            ;
                PUSH_


                bcf             INTCON,2        ;resets interrupt bit

                decfsz          ISR_TMR,f       ;prescaler (/256)
                goto            _ISRNOSEC

                decfsz          ISR_TMR1,f
                goto            _ISRNOSEC

                movlw           C_ISRTIME
                movwf           ISR_TMR1

                bsf             F_1HZTRIG       ;bit ist zwischenhndler zwischen interrupt und main

_ISRNOSEC

                POP_

		retfie

;******************************************************************************
;*************************************************** END OF INTERRUPT FUNCTIONS

;***************************************************************** Look Up ****
;******************************************************************************




;********************************7seg decoder**********************************
;
;       Bit0 db/nc
;       Bit1 c
;       Bit2 d
;       Bit3 e
;       
;       Bit4 g
;       Bit5 f
;       Bit6 a
;       Bit7 b
;       
;
;******************************************************************************

		org		0x03DB		
_7SEG
		movlw		h'03'
		movwf		PCLATH
		movf		VAL,w
                addlw           h'E0'           ;sub 20h
		addwf		PCL,f

						;asci 20h

		retlw		h'00'		;Leerzeichen		
		retlw		h'83'		;!
		retlw		h'A0'		;"
		retlw		h'54'		;#
		retlw		h'72'		;$
		retlw		h'66'		;%
		retlw		h'FF'		;&
		retlw		h'80'		;'
		retlw		h'6C'		;( = [
		retlw		h'C6'		;) = ]
		retlw		h'BA'		;* = H
		retlw		h'92'		;+ = {
		retlw		h'02'		;,
		retlw		h'10'		;-
		retlw		h'01'		;.
		retlw		h'98'		;/ 

						;asci 30

		retlw		h'EE'		;0		
		retlw		h'82'		;1
		retlw		h'DC'		;2
		retlw		h'D6'		;3
		retlw		h'B2'		;4
		retlw		h'76'		;5
		retlw		h'7E'		;6
		retlw		h'C2'		;7
		retlw		h'FE'		;8
		retlw		h'F6'		;9
		retlw		h'50'		;:
		retlw		h'51'		;;
		retlw		h'1C'		;<
		retlw		h'50'		;==:
		retlw		h'16'		;>
		retlw		h'D9'		;?


;********************************************************** end of 7 seg decode
;******************************************************************************

END

